@@ -38,15 +38,10 @@ __inline static void Rhpc_mpi_lapply_seq_exit(int procs, MPI_Comm comm)
   SET_CMD(cmde, CMD_NAME_LAPPLY_SEQ , SUBCMD_EXIT, 0, 0 );
   for (i=1; i < procs ; i++){
     DPRINT("worker exit rank=%ld\n",i);
-    if(SYNC){
-      _M(MPI_Send(cmde,   (int)CMDLINESZ, MPI_INT, i, RHPC_CTRL_TAG, comm));
-    }else{
-      _M(MPI_Isend(cmde,   (int)CMDLINESZ, MPI_INT, i, RHPC_CTRL_TAG, comm, &request[i-1]));
-    }
-  }
-  if(!SYNC){
-    _M(MPI_Waitall(procs-1, request, status));
+    _M(MPI_Isend(cmde,   (int)CMDLINESZ, MPI_INT, i, RHPC_CTRL_TAG, comm, &request[i-1]));
   }
+  _M(MPI_Waitall(procs-1, request, status));
+
   Free(request);
   Free(status);
 }
@@ -58,7 +53,6 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
 					      MPI_Comm comm)
 {
   R_xlen_t i,j;
-  int errorOccurred=0;
 
   SEXP Xsel_l;
   SEXP Xsel;
@@ -83,14 +77,7 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
   
   for (i=1; i < procs ; i++){
     workers[i]=xlength(VECTOR_ELT(XL,i-1));
-    if(SERMODE){
-      Xsel_l = LCONS(install("serialize"),CONS(VECTOR_ELT(XL,i-1),CONS(R_NilValue, CONS(ScalarLogical(FALSE), CONS(R_NilValue, CONS(R_NilValue, R_NilValue))))));
-      Xsel_l = LCONS(install(".Internal"), CONS(Xsel_l, R_NilValue));
-      REPROTECT(Xsel_l,ix1);
-      REPROTECT(Xsel= R_tryEval(Xsel_l, R_GlobalEnv, &errorOccurred), ix2);
-    }else{/* SERMODE=0 */
-      REPROTECT(Xsel= Rhpc_serialize(VECTOR_ELT(XL,i-1)), ix2);
-    }
+    REPROTECT(Xsel= Rhpc_serialize(VECTOR_ELT(XL,i-1)), ix2);
     SET_VECTOR_ELT(sendlist,i-1, Xsel);
   }
   
@@ -120,33 +107,19 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
 	mydump((void*)cmds,CMDLINESZ*sizeof(int));
       */
       for( j = 0 ; j< cnts ; j++){
-	if(SYNC){
-	  _M(MPI_Send(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*j,
-		      (int)RHPC_SPLIT_SIZE, MPI_CHAR,
-		      (int)i, TAGCAL(j),    comm));
-	}else{
-	  _M(MPI_Isend(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*j,
-		       (int)RHPC_SPLIT_SIZE, MPI_CHAR,
-		       (int)i, TAGCAL(j),    comm, &request[sendcalls]));
-	}
+	_M(MPI_Isend(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*j,
+		     (int)RHPC_SPLIT_SIZE, MPI_CHAR,
+		     (int)i, TAGCAL(j),    comm, &request[sendcalls]));
 	sendcalls++;
       }
       if ( mods != 0 ){
-	if(SYNC){
-	  _M(MPI_Send(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*cnts,
-		       (int)mods,            MPI_CHAR,
-		       (int)i, TAGCAL(cnts), comm));
-	}else{
-	  _M(MPI_Isend(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*cnts,
-		       (int)mods,            MPI_CHAR,
-		       (int)i, TAGCAL(cnts), comm, &request[sendcalls]));
-	}
+	_M(MPI_Isend(RAW(VECTOR_ELT(sendlist,i-1))+RHPC_SPLIT_SIZE*cnts,
+		     (int)mods,            MPI_CHAR,
+		     (int)i, TAGCAL(cnts), comm, &request[sendcalls]));
 	sendcalls++;
       }
     }
-    if(!SYNC){
-      _M(MPI_Waitall(sendcalls, request, status));
-    }
+    _M(MPI_Waitall(sendcalls, request, status));
     Free(request);
     Free(status);
   } /* send block === end ============================================== */
@@ -159,8 +132,6 @@ __inline static void Rhpc_mpi_lapply_seq_send(R_xlen_t *workers,
 SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
 {
   R_xlen_t i;
-  int errorOccurred=0;
-
 
   MPI_Comm comm;
   int procs;
@@ -181,6 +152,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
   SEXP          outlist;
   PROTECT_INDEX outlist_l_ix;
   PROTECT_INDEX outlist_ix;
+  SEXP          names = getAttrib(X, R_NamesSymbol);
 
   SEXP indata;
   SEXP uns;
@@ -209,14 +181,9 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
 
   /* -------- common start */
   /* serialize */
-  if(SERMODE){
-    l_out=LCONS(install("serialize"),CONS(args,CONS(R_NilValue, CONS(ScalarLogical(FALSE), CONS(R_NilValue, CONS(R_NilValue, R_NilValue))))));
-    PROTECT(l_out=LCONS(install(".Internal"), CONS(l_out, R_NilValue)));
-    PROTECT(out=R_tryEval(l_out, R_GlobalEnv, &errorOccurred));
-  }else{
-    PROTECT(l_out);
-    PROTECT(out=Rhpc_serialize(args));
-  }
+  PROTECT(names);
+  PROTECT(l_out);
+  PROTECT(out=Rhpc_serialize(args));
 
 
   /* cmd send */
@@ -250,6 +217,8 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
   PROTECT_WITH_INDEX(indata   =R_NilValue,               &indata_ix);
   PROTECT_WITH_INDEX(uns_l    =R_NilValue,               &uns_l_ix);
   PROTECT_WITH_INDEX(uns      =R_NilValue,               &uns_ix);
+
+  if(!isNull(names)) setAttrib(outlist, R_NamesSymbol, names);
   
   Rhpc_mpi_lapply_seq_send(workers, procs, X, comm);
 
@@ -263,6 +232,7 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
       if(stat.MPI_SOURCE != 0){
 	int cmdr[CMDLINESZ];
 	int wkr = stat.MPI_SOURCE;
+	int wkrix = 0;
 	int  cmdmainr = 0;
 	int  cmdsubr  = 0;
 	R_xlen_t cntr = 0;
@@ -281,54 +251,36 @@ SEXP Rhpc_mpi_lapply_seq(SEXP cl, SEXP X, SEXP args)
 	  MPI_Request *request = Calloc(msgcnt, MPI_Request);
 	  MPI_Status  *status  = Calloc(msgcnt, MPI_Status);
 	  for(i=0;i<cntr;i++){
-	    if(SYNC){
-	      _M(MPI_Recv(RAW(indata)+RHPC_SPLIT_SIZE*i, 
-			   (int)RHPC_SPLIT_SIZE, MPI_CHAR,
-			   wkr, TAGCAL(i),    comm, &status[calls]));
-	    }else{
-	      _M(MPI_Irecv(RAW(indata)+RHPC_SPLIT_SIZE*i, 
-			   (int)RHPC_SPLIT_SIZE, MPI_CHAR,
-			   wkr, TAGCAL(i),    comm, &request[calls]));
-	    }
+	    _M(MPI_Irecv(RAW(indata)+RHPC_SPLIT_SIZE*i, 
+			 (int)RHPC_SPLIT_SIZE, MPI_CHAR,
+			 wkr, TAGCAL(i),    comm, &request[calls]));
 	    calls++;
 	  }
 	  if ( modr != 0 ){
-	    if(SYNC){
-	      _M(MPI_Recv(RAW(indata)+RHPC_SPLIT_SIZE*cntr,
-			   (int)modr,            MPI_CHAR,
-			   wkr, TAGCAL(cntr), comm, &status[calls]));
-	    }else{
-	      _M(MPI_Irecv(RAW(indata)+RHPC_SPLIT_SIZE*cntr,
-			   (int)modr,            MPI_CHAR,
-			   wkr, TAGCAL(cntr), comm, &request[calls]));
-	    }
+	    _M(MPI_Irecv(RAW(indata)+RHPC_SPLIT_SIZE*cntr,
+			 (int)modr,            MPI_CHAR,
+			 wkr, TAGCAL(cntr), comm, &request[calls]));
 	    calls++;
 	  }
-	  if(!SYNC){
-	    _M(MPI_Waitall(calls, request, status));
-	  }
+	  _M(MPI_Waitall(calls, request, status));
 	  Free(status);
 	  Free(request);
 	}
 	
-	if(SERMODE){
-	  uns_l=LCONS(install("unserialize"),CONS(indata, CONS(R_NilValue,R_NilValue)));
-	  uns_l=LCONS(install(".Internal"),  CONS(uns_l,                  R_NilValue));
- 
-	  REPROTECT(uns_l, uns_l_ix);
-	  REPROTECT(uns  = R_tryEval(uns_l, R_GlobalEnv, &errorOccurred),uns_ix);
-	}else{ /* SERMODE=0 */
-	  REPROTECT(uns  = Rhpc_unserialize(indata),uns_ix);
+	REPROTECT(uns  = Rhpc_unserialize(indata),uns_ix);
+
+	for(wkrix=0 ; wkrix<xlength(uns); wkrix++){
+	  SET_VECTOR_ELT(outlist, (wkr-1)+wkrix*(procs-1), VECTOR_ELT(uns,wkrix) );
+	  REPROTECT(outlist,outlist_ix);
 	}
-	SET_VECTOR_ELT(outlist, workersix[wkr], uns );
-	workers[wkr]--;
+	workers[wkr]-=xlength(uns);
 	workersix[wkr]+=procs-1;
 	DPRINT("finish rank=%d ix=%d\n", wkr, workersix[wkr] );
       }
     }
   }
   Rhpc_mpi_lapply_seq_exit(procs, comm);
-  UNPROTECT(7);
+  UNPROTECT(8);
   Free(workers);
   Free(workersix);
 
