@@ -54,8 +54,10 @@ static void Rhpc_worker_lapply_LB(int *cmd)
    }
 
   /* unserialize */
+  fun_arg=Rhpc_unserialize(data);
+  UNPROTECT(1);
+  PROTECT(fun_arg);
   PROTECT(l_fun_arg=R_NilValue);
-  PROTECT(fun_arg=Rhpc_unserialize(data));
 
   /* find function */
   fun = VECTOR_ELT(fun_arg,0);
@@ -105,7 +107,7 @@ static void Rhpc_worker_lapply_LB(int *cmd)
     */
     GET_CMD(cmdx, &getx, &getsubx, &cntx, &modx);
     if( getsubx == SUBCMD_EXIT ){
-      UNPROTECT(6);
+      UNPROTECT(5);
       return;
     }
     lenx = RHPC_SPLIT_SIZE * cntx + modx;
@@ -251,7 +253,7 @@ static void Rhpc_worker_lapply_LB(int *cmd)
       Free(status);
       DPRINT("send data wait end\n");
     }
-    UNPROTECT(13);
+    UNPROTECT(12);
   }
   return;
 }
